@@ -1,4 +1,4 @@
-/*  P A S - P. WOLFERS SOFTWARE: V2.1-E OF 31-AUG-2009 
+/*  P A S - P. WOLFERS SOFTWARE: V3.0-A OF 31-OCT-2014 
   ***  PASCAL II implementation in C. *** */
 
 /* Set specific Macro setting. */
@@ -52,32 +52,31 @@ static Blk( Rd, 120 ) = {
 };
 
 /* Define the local static Initialized section. */
-static Blk( Ri, 392 ) = {
+static Blk( Ri, 384 ) = {
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0, 255, 255, 255, 255,   1,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-255, 255, 255, 255,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 144,  32,   0,   2,
- 32,  32,  32,  32,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0, 144,  32,   0,   2,  32,  32,  32,  32,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   1,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 145,   0,   0,   4,
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   1,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,
   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0, 146,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0, 145,   0,   0,   4,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  2,   0,   0,   0,   1,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,
+  1,   0,   0,   0,   1,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 146,   0,   0,   0,
-  2,   0,   0,   0,   0,   0,   0,   0
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   1,   0,   0,   0,
+  0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0, 146,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0
 };
 
 /* Define the local static section. */
@@ -180,6 +179,25 @@ void PAS__ins_proc(V *F_pp, void F_pr(V F_fp));
 void PAS__exe_proc(V F_pp, V F_fp);
 
 
+/* Procedure/Function : " pas_$new "  */
+V PAS__NEW(SL F_size){
+  register V Ret;
+
+  /* Code of procedure/function */
+  Ret=PAS__MEM_ALLOC(F_size);
+  return(Ret);
+}
+
+
+/* Procedure/Function : " pas_$dispose "  */
+void PAS__DISPOSE(V *F_ptr){
+
+  /* Code of procedure/function */
+  PAS__MEM_FREE((*F_ptr));
+  (*F_ptr)=NULL;
+}
+
+
 /* Procedure/Function : " alloc_new_heap "  */
 void PAS__STD_PRC_00000000(SL F_sz){
   /* Local variable stored in C variable */
@@ -187,23 +205,22 @@ void PAS__STD_PRC_00000000(SL F_sz){
   V Rv1;
 
   /* Code of procedure/function */
-  F_sz=F_sz+3;
   if (PAS__heap_incr<128) PAS__heap_incr=128;
+  F_sz=F_sz+20;
   F_sz=(((F_sz+PAS__heap_incr)-1)/PAS__heap_incr)*PAS__heap_incr;
-  Ri.ub[0]=0;
+  F_sz=(F_sz+7)/8;
   Rv0=F_sz;
   Rv1=PAS__NEW((Rv0+1)*8+24);
   ((SL *)Rv1)[0]=Rv0;
-  Ri.ub[0]=1;
   ((SL *)Rv1)[1]=0;
-  ((V *)Rv1)[1]=Ri.v[1];
-  ((V *)Rv1)[2]=Ri.v[2];
-  Ri.v[1]=Rv1;
+  ((V *)Rv1)[1]=Ri.v[0];
+  ((V *)Rv1)[2]=Ri.v[1];
+  Ri.v[0]=Rv1;
 }
 
 
 /* Procedure/Function : " heap_alloc "  */
-V PAS__STD_heap_alloc(SL F_size){
+V PAS__HEAP_ALLOC(SL F_size){
   /* Local variable stored in C variable */
   register SL Rv0;
   register V Rv1;
@@ -214,56 +231,32 @@ V PAS__STD_heap_alloc(SL F_size){
   register V Ret;
 
   /* Code of procedure/function */
-  if (Ri.v[1]==NULL) {
-    Ri.ub[0]=0;
+  if (Ri.v[0]==NULL) {
     if (PAS__heap_isiz<128) PAS__heap_isiz=128;
     Rv0=PAS__heap_isiz;
-    Ri.v[1]=PAS__NEW((Rv0+1)*8+24);
-    ((SL *)Ri.v[1])[0]=Rv0;
-    Rv1=Ri.v[1];
+    Ri.v[0]=PAS__NEW((Rv0+1)*8+24);
+    ((SL *)Ri.v[0])[0]=Rv0;
+    Rv1=Ri.v[0];
     ((SL *)Rv1)[1]=0;
     ((V *)Rv1)[1]=NULL;
     ((V *)Rv1)[2]=NULL;
-    Ri.v[2]=NULL;
-    Ri.ub[0]=1;
+    Ri.v[1]=NULL;
   }
-  Rv2=((SL *)Ri.v[1])[1];
+  Rv2=((SL *)Ri.v[0])[1];
   Rv3=(F_size+7)/8;
   Rv4=Rv2+Rv3;
-  if (Rv4>=((SL *)Ri.v[1])[0]) {
+  if (Rv4>=((SL *)Ri.v[0])[0]) {
     PAS__STD_PRC_00000000(Rv3);
     Rv4=Rv3;
     Rv2=0;
   }
-  Rv5=Ri.v[1];
+  Rv5=Ri.v[0];
   ((SL *)Rv5)[1]=Rv4;
   Ret=cc_wild_pointer((((G *)Rv5)+Rv2+3));
   return(Ret);
 }
 
 
-/* Procedure/Function : " pas_$new "  */
-V PAS__NEW(SL F_size){
-  register V Ret;
-
-  /* Code of procedure/function */
-  if (Ri.ub[0]&&Ri.v[1]!=NULL) Ret=PAS__STD_heap_alloc(F_size);
-  else Ret=PAS__MEM_ALLOC(F_size);
-  return(Ret);
-}
-
-
-/* Procedure/Function : " pas_$dispose "  */
-void PAS__DISPOSE(V *F_ptr){
-
-  /* Code of procedure/function */
-  if (!Ri.ub[0]) {
-    PAS__MEM_FREE((*F_ptr));
-    (*F_ptr)=NULL;
-  }
-}
-
-
 /* Procedure/Function : " pas_$mark "  */
 void PAS__MARK(){
   /* Local variable stored in C variable */
@@ -273,22 +266,21 @@ void PAS__MARK(){
   register V Rv3;
 
   /* Code of procedure/function */
-  Ri.ub[0]=1;
-  if (Ri.v[1]!=NULL) {
-    Rv0=Ri.v[1];
-    Rv1=((SL *)Ri.v[1])[1];
+  if (Ri.v[0]!=NULL) {
+    Rv0=Ri.v[0];
+    Rv1=((SL *)Ri.v[0])[1];
   }
   else {
     Rv0=NULL;
     Rv1=0;
   }
-  Rv2=PAS__NEW(20);
+  Rv2=PAS__HEAP_ALLOC(20);
   Rv3=Rv2;
-  ((V *)Rv3)[0]=Ri.v[2];
-  if (Rv0==NULL) ((V *)Rv3)[1]=Ri.v[1];
+  ((V *)Rv3)[0]=Ri.v[1];
+  if (Rv0==NULL) ((V *)Rv3)[1]=Ri.v[0];
   else((V *)Rv3)[1]=Rv0;
   ((SL *)Rv3)[4]=Rv1;
-  Ri.v[2]=Rv2;
+  Ri.v[1]=Rv2;
 }
 
 
@@ -302,37 +294,22 @@ void PAS__RELEASE(){
   V Rv4;
 
   /* Code of procedure/function */
-  if ((Ri.ub[0]&&Ri.v[1]!=NULL)&&Ri.v[2]!=NULL) {
-    Rv0=Ri.v[2];
+  if (Ri.v[0]!=NULL&&Ri.v[1]!=NULL) {
+    Rv0=Ri.v[1];
     Rv1=((V *)Rv0)[0];
     Rv2=((V *)Rv0)[1];
     Rv3=((SL *)Rv0)[4];
-    if (Rv2!=NULL&&Rv2!=Ri.v[1]) {
-      Ri.ub[0]=0;
-      while (Rv2!=Ri.v[1]){
-        Rv4=((V *)Ri.v[1])[1];
-        PAS__DISPOSE(Ri.v+1);
-        Ri.v[1]=Rv4;
-      }
-      Ri.ub[0]=1;
+    if (Rv2!=NULL&&Rv2!=Ri.v[0]) while (Rv2!=Ri.v[0]){
+      Rv4=((V *)Ri.v[0])[1];
+      PAS__DISPOSE(Ri.v);
+      Ri.v[0]=Rv4;
     }
-    ((SL *)Ri.v[1])[1]=Rv3;
-    Ri.v[2]=Rv1;
+    ((SL *)Ri.v[0])[1]=Rv3;
+    Ri.v[1]=Rv1;
   }
 }
 
 
-/* Procedure/Function : " pas__heap_mode "  */
-UB PAS__HEAP_MODE(UB F_flg){
-  register UB Ret;
-
-  /* Code of procedure/function */
-  Ret=Ri.ub[0];
-  Ri.ub[0]=F_flg;
-  return(Ret);
-}
-
-
 /* Procedure/Function : " pas__get_env "  */
 SL PAS__GET_ENV_ARRAY(V *F_ptr, V F_src){
   /* Local variable stored in C variable */
@@ -481,15 +458,15 @@ void PAS__STD_vread_end(V F_fp){
 
 
   /* Code of procedure/function */
-  Ra.v[0]=Ri.v[7];
-  if (Ri.sl[23]>0) {
-    if (Ri.v[7]!=NULL) ((SL *)Ra.v[0])[0]=(Ri.sl[22]-Ri.sl[23])+1;
+  Ra.v[0]=Ri.v[6];
+  if (Ri.sl[21]>0) {
+    if (Ri.v[6]!=NULL) ((SL *)Ra.v[0])[0]=(Ri.sl[20]-Ri.sl[21])+1;
   }
   else {
-    Ri.sl[23]=0;
-    Ri.v[3]=Ri.s+112;
-    Ri.ul[27]=Ri.ul[27]|184549376;
-    if (Ri.v[7]!=NULL) ((SL *)Ra.v[0])[0]=-1;
+    Ri.sl[21]=0;
+    Ri.v[2]=Ri.s+104;
+    Ri.ul[25]=Ri.ul[25]|184549376;
+    if (Ri.v[6]!=NULL) ((SL *)Ra.v[0])[0]=-1;
   }
 }
 
@@ -502,15 +479,15 @@ void PAS__VREAD_INIT(V F_str, SL F_sz, SL *F_ip){
   /* Code of procedure/function */
   if (((V) F_ip)!=NULL) Rv0=(*F_ip);
   else Rv0=1;
-  Ri.v[3]=(((C *)F_str)+Rv0-1);
-  Ri.v[4]=F_str;
-  Ri.v[7]=((V) F_ip);
-  Ri.v[9]=F_str;
-  PAS__ins_proc(Ri.v+8,PAS__STD_vread_end);
-  Ri.sl[22]=F_sz;
-  Ri.sl[23]=(F_sz-Rv0)+1;
-  Ri.ul[27]=8337;
-  PAS__curr_iptr=Ri.s+24;
+  Ri.v[2]=(((C *)F_str)+Rv0-1);
+  Ri.v[3]=F_str;
+  Ri.v[6]=((V) F_ip);
+  Ri.v[8]=F_str;
+  PAS__ins_proc(Ri.v+7,PAS__STD_vread_end);
+  Ri.sl[20]=F_sz;
+  Ri.sl[21]=(F_sz-Rv0)+1;
+  Ri.ul[25]=8337;
+  PAS__curr_iptr=Ri.s+16;
 }
 
 
@@ -524,17 +501,17 @@ void PAS__STD_twrite_end(V F_fp){
   register int Rf0;
 
   /* Code of procedure/function */
-  Ra.v[0]=Ri.v[7];
-  if (Ri.v[7]!=NULL) ((SL *)Ra.v[0])[0]=Ri.sl[23];
+  Ra.v[0]=Ri.v[6];
+  if (Ri.v[6]!=NULL) ((SL *)Ra.v[0])[0]=Ri.sl[21];
   else {
-    Rv0=Ri.sl[23]+1;
-    for( Rf0=(Ri.sl[22]+1)-(Ri.sl[23]+1);Rf0>0;Rf0-- ) {
-      ((C *)Ri.v[4])[Rv0]=32;
+    Rv0=Ri.sl[21]+1;
+    for( Rf0=(Ri.sl[20]+1)-(Ri.sl[21]+1);Rf0>0;Rf0-- ) {
+      ((C *)Ri.v[3])[Rv0]=32;
       Rv0++;
     }
   }
-  if (33554432&Ri.ul[27]) PAS__ERROR(54);
-  else Ri.ul[27]=Ri.ul[27]|167772160;
+  if (33554432&Ri.ul[25]) PAS__ERROR(54);
+  else Ri.ul[25]=Ri.ul[25]|167772160;
 }
 
 
@@ -546,15 +523,15 @@ void PAS__TWRITE_INIT(V F_str, SL F_sz, SL *F_ip){
   /* Code of procedure/function */
   if (((V) F_ip)!=NULL) Rv0=(*F_ip);
   else Rv0=0;
-  Ri.v[3]=(((C *)F_str)+(Rv0+1)-1);
-  Ri.v[4]=F_str;
-  Ri.v[7]=((V) F_ip);
-  Ri.v[9]=F_str;
-  PAS__ins_proc(Ri.v+8,PAS__STD_twrite_end);
-  Ri.sl[22]=F_sz;
-  Ri.sl[23]=Rv0;
-  Ri.ul[27]=8338;
-  PAS__curr_optr=Ri.s+24;
+  Ri.v[2]=(((C *)F_str)+(Rv0+1)-1);
+  Ri.v[3]=F_str;
+  Ri.v[6]=((V) F_ip);
+  Ri.v[8]=F_str;
+  PAS__ins_proc(Ri.v+7,PAS__STD_twrite_end);
+  Ri.sl[20]=F_sz;
+  Ri.sl[21]=Rv0;
+  Ri.ul[25]=8338;
+  PAS__curr_optr=Ri.s+16;
 }
 
 
@@ -562,9 +539,9 @@ void PAS__TWRITE_INIT(V F_str, SL F_sz, SL *F_ip){
 void PAS__STD_vwrite_end(V F_fp){
 
   /* Code of procedure/function */
-  ((UB *)Ri.v[9])[1]=Ri.sl[23];
-  if (33554432&Ri.ul[27]) PAS__ERROR(54);
-  else Ri.ul[27]=Ri.ul[27]|167772160;
+  ((UB *)Ri.v[8])[1]=Ri.sl[21];
+  if (33554432&Ri.ul[25]) PAS__ERROR(54);
+  else Ri.ul[25]=Ri.ul[25]|167772160;
 }
 
 
@@ -572,20 +549,20 @@ void PAS__STD_vwrite_end(V F_fp){
 void PAS__VWRITE_INIT(V F_str, UB F_bini){
 
   /* Code of procedure/function */
-  Ri.v[4]=F_str+2;
-  Ri.v[9]=F_str;
-  PAS__ins_proc(Ri.v+8,PAS__STD_vwrite_end);
-  Ri.sl[22]=((UB *)F_str)[0];
+  Ri.v[3]=F_str+2;
+  Ri.v[8]=F_str;
+  PAS__ins_proc(Ri.v+7,PAS__STD_vwrite_end);
+  Ri.sl[20]=((UB *)F_str)[0];
   if (F_bini) {
-    Ri.v[3]=F_str+2;
-    Ri.sl[23]=0;
+    Ri.v[2]=F_str+2;
+    Ri.sl[21]=0;
   }
   else {
-    Ri.v[3]=(((C *)F_str)+(((UB *)F_str)[1]+1)+1);
-    Ri.sl[23]=((UB *)F_str)[1];
+    Ri.v[2]=(((C *)F_str)+(((UB *)F_str)[1]+1)+1);
+    Ri.sl[21]=((UB *)F_str)[1];
   }
-  Ri.ul[27]=8338;
-  PAS__curr_optr=Ri.s+24;
+  Ri.ul[25]=8338;
+  PAS__curr_optr=Ri.s+16;
 }
 
 
@@ -848,7 +825,7 @@ void PAS__STD_get_file_elem(V F_fp){
           }
           else {
             ((SL *)Rv0)[17]=0;
-            if (Rv1==0) ((V *)Rv0)[0]=Ri.s+112;
+            if (Rv1==0) ((V *)Rv0)[0]=Ri.s+104;
             ((UL *)Rv0)[21]=(((UL *)Rv0)[21]|67108864)&~16777216;
           }
         }
@@ -1144,7 +1121,7 @@ void PAS__WRITE_EOLN(){
   /* Code of procedure/function */
   if (PAS__curr_optr!=NULL) {
     Rv0=PAS__curr_optr;
-    if (32&((UL *)Rv0)[21]||PAS__curr_optr==Ri.s+24) {
+    if (32&((UL *)Rv0)[21]||PAS__curr_optr==Ri.s+16) {
       if (8192&((UL *)Rv0)[21]) PAS__exe_proc(((V *)Rv0)[5],PAS__curr_optr);
       else PAS_put_buffer(PAS__curr_optr);
     }
@@ -1922,36 +1899,36 @@ void PAS__INIT(SL F_parc, V F_parv, V F_penv){
   /* Code of procedure/function */
   Ra.ub[0]=255;
   CC_TRAP_INIT();
-  PAS__f_input=Ri.s+120;
-  PAS__f_output=Ri.s+208;
-  PAS__f_error=Ri.s+296;
+  PAS__f_input=Ri.s+112;
+  PAS__f_output=Ri.s+200;
+  PAS__f_error=Ri.s+288;
   if (PAS__GETENV(Ra.s,Rd.s+47)<=0) PAS__SETENV(Rd.s+47,Rd.s+51,1);
   if (PAS__GETENV(Ra.s,Rd.s+61)<=0) PAS__SETENV(Rd.s+61,Rd.s+65,1);
-  Ri.sl[44]=0;
-  Ri.sl[45]=1;
-  Ri.sl[46]=256;
-  Ri.ul[51]=67109009;
-  if (isatty(Ri.sl[44])!=0) PAS__STD_PRC_00000001(Ri.sl[44],Ra.s);
+  Ri.sl[42]=0;
+  Ri.sl[43]=1;
+  Ri.sl[44]=256;
+  Ri.ul[49]=67109009;
+  if (isatty(Ri.sl[42])!=0) PAS__STD_PRC_00000001(Ri.sl[42],Ra.s);
   else PAS__STR_TO_STR(Ra.s,Rd.s+76);
-  PAS__STD_init_a_file(Ri.s+120,Ra.s);
-  Ri.sl[66]=1;
-  Ri.sl[67]=1;
-  Ri.sl[68]=256;
-  Ri.ul[73]=146;
-  if (isatty(Ri.sl[66])!=0) PAS__STD_PRC_00000001(Ri.sl[66],Ra.s);
+  PAS__STD_init_a_file(Ri.s+112,Ra.s);
+  Ri.sl[64]=1;
+  Ri.sl[65]=1;
+  Ri.sl[66]=256;
+  Ri.ul[71]=146;
+  if (isatty(Ri.sl[64])!=0) PAS__STD_PRC_00000001(Ri.sl[64],Ra.s);
   else PAS__STR_TO_STR(Ra.s,Rd.s+87);
-  PAS__STD_init_a_file(Ri.s+208,Ra.s);
-  Ri.sl[88]=2;
-  Ri.sl[89]=1;
-  Ri.sl[90]=256;
-  Ri.ul[95]=146;
-  if (isatty(Ri.sl[88])!=0) PAS__STD_PRC_00000001(Ri.sl[88],Ra.s);
+  PAS__STD_init_a_file(Ri.s+200,Ra.s);
+  Ri.sl[86]=2;
+  Ri.sl[87]=1;
+  Ri.sl[88]=256;
+  Ri.ul[93]=146;
+  if (isatty(Ri.sl[86])!=0) PAS__STD_PRC_00000001(Ri.sl[86],Ra.s);
   else PAS__STR_TO_STR(Ra.s,Rd.s+99);
-  PAS__STD_init_a_file(Ri.s+296,Ra.s);
-  Ri.sl[20]=-1;
-  Ri.sl[21]=1;
-  Ri.sl[22]=0;
-  Ri.ul[27]=33562768;
+  PAS__STD_init_a_file(Ri.s+288,Ra.s);
+  Ri.sl[18]=-1;
+  Ri.sl[19]=1;
+  Ri.sl[20]=0;
+  Ri.ul[25]=33562768;
   PAS__argc=F_parc;
   Rv0=0;
   for( Rf1=(F_parc-1)+1;Rf1>0;Rf1-- ) {
@@ -1970,7 +1947,7 @@ void PAS__INIT(SL F_parc, V F_parv, V F_penv){
         Rv4++;
       }
     }
-    else PAS__argv.v[Rv0]=Ri.s+384;
+    else PAS__argv.v[Rv0]=Ri.s+376;
     Rv0++;
   }
   Rv5=F_parc;
